{% extends "layout.html" %}
{% block body %}
<div class="jumbotron" id="landing-info">
    <h1>Something went wrong!</h1>
    <hr>
    <h2>Error {{ error.code }}</h2>
    <p>{{ error.description }}</p>
</div>
{% endblock %}
