<nav class="navbar navbar-default navbar-fixed-top navbar-inverse">
    <div class="container">
        <div class="navbar-header">
            <button type="button" class="navbar-toggle collapsed" data-toggle="collapse" data-target="#navbar"
                    aria-expanded="false" aria-controls="navbar">
                <span class="sr-only">Toggle navigation</span>
                <span class="icon-bar"></span>
                <span class="icon-bar"></span>
                <span class="icon-bar"></span>
            </button>
            <a class="navbar-logo" href="{{ url_for('index') }}">
                <span class="navbar-brand"><span class="octicon octicon-radio-tower"></span> Labelord server</span>
            </a>
        </div>
        <div id="navbar" class="navbar-collapse collapse">
            <ul class="nav navbar-nav navbar-right">
                <li>
                    <a href="{{ url_for('index') }}"><span class="octicon octicon-home"></span> Home</a>
                </li>
            </ul>
        </div><!--/.nav-collapse -->
    </div>
</nav>
