
DEFAULT_SUCCESS_RETURN = 0
DEFAULT_ERROR_RETURN = 10
NO_GH_TOKEN_RETURN = 3
GH_ERROR_RETURN = {
    401: 4,
    404: 5
}
NO_LABELS_SPEC_RETURN = 6
NO_REPOS_SPEC_RETURN = 7
NO_WEBHOOK_SECRET_RETURN = 8

DEFAULT_CONFIG_FILE = './config.cfg'
